#include "scene.h"

#include <QGraphicsSceneMouseEvent>
#include <QGraphicsPixmapItem>
#include <QGraphicsRectItem>
#include <QDebug>


SceneSheme::SceneSheme(QObject* parent) : QGraphicsScene(parent)
{
    m_colorSelectionPen = new QColor(158,182,255,200);
    m_colorSelectionBrush = new QColor(158,182,255,96);
    setSceneRect(0, 0, 50, 50);
}

void SceneSheme::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    if (event->button() & Qt::LeftButton)
    {
        updateSceneRect(event);
        m_leftMouseButtonPressed = true;
        setPreviousPosition(event->scenePos());
        m_AreaSelection = new AreaSelector();
        m_AreaSelection->setPos(event->scenePos());
        addItem(m_AreaSelection);
        update();
    }

    QGraphicsScene::mousePressEvent(event);
}

void SceneSheme::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    if (m_leftMouseButtonPressed)
    {
        updateSceneRect(event);
        auto dx = event->scenePos().x() - m_previousPosition.x();
        auto dy = event->scenePos().y() - m_previousPosition.y();
        m_AreaSelection->setSelectedArea(QRectF((dx > 0) ? 0 : dx, (dy > 0) ? 0 : dy, qAbs(dx), qAbs(dy)));
        update();
    }
    QGraphicsScene::mouseMoveEvent(event);
}

void SceneSheme::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
    if (event->button() & Qt::LeftButton)
    {
        m_leftMouseButtonPressed = false;
        updateSceneRect(event);
        update();
    }
    QGraphicsScene::mouseReleaseEvent(event);
}

void SceneSheme::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
{
    qDebug()<<"doubleclick";
    QGraphicsScene::mouseDoubleClickEvent(event);
}

void SceneSheme::updateSceneRect(QGraphicsSceneMouseEvent *event)
{
    QRectF sceneRect = this->sceneRect();
    double x = event->pos().x();
    double y = event->pos().y();
    if (x < sceneRect.left()) sceneRect.setLeft(x);
    if (x > (sceneRect.right())) sceneRect.setRight(x);
    if (y < sceneRect.top()) sceneRect.setTop(y);
    if (y > (sceneRect.bottom())) sceneRect.setBottom(y);
    setSceneRect(sceneRect);
}

void SceneSheme::setPreviousPosition(const QPointF previousPosition)
{
    if (m_previousPosition == previousPosition)
        return;

    m_previousPosition = previousPosition;
    emit previousPositionChanged(m_previousPosition);
}

QPointF SceneSheme::previousPosition() const
{
    return m_previousPosition;
}
